/*
*********************************************************************************************************
*                                              EE4S20 Exercise 3
*                                               Semaphores
*
*                           The application program has been rewritten from the base code provided
*                           by Micrium. 
*
*                          (c) Copyright 2003-2006; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
*                                             Renesas M16C
*                                                on the
*                                      RKM 16C62P Evaluation Board
*
* Filename      : app.c
* Version       : V1.00
* Programmer(s) : Ralf Patz   (original code: Brian Nagel)
*********************************************************************************************************
*/

#include <includes.h>

/*
*********************************************************************************************************
*                                           #DEFINE CONSTANTS
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                            LOCAL VARIABLES
*********************************************************************************************************
*/

static  OS_STK          AppTaskStartStk[APP_TASK_START_STK_SIZE];
static  OS_STK          AppTaskLEDFlashStk[APP_TASK_LED_FLASH_STK_SIZE];
static  OS_STK          AppTaskLEDReadStk[APP_TASK_LED_READ_STK_SIZE];
static  OS_EVENT        *MyLEDSem; //Declare the global variable


/*
*********************************************************************************************************
*                                         LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void            AppTaskCreate               (void);
static  void            AppTaskStart                (void *p_arg);
static  void            AppTaskLEDFlash             (void *p_arg);
static  void            AppTaskLEDRead              (void *p_arg);


#if OS_VIEW_MODULE > 0
static  void            AppTerminalRx       (CPU_INT08U rx_data);
#endif

static  void            AppFormatDec        (CPU_INT08U *s, CPU_INT32U value, CPU_INT08U digits);

/*
*********************************************************************************************************
*                                                main()
*
* Description : This is the standard entry point for C code.  It is assumed that your code will call
*               main() once you have performed all necessary 68HC12 and C initialization.
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/

void  main (void)
{
    CPU_INT08U  err;


    BSP_IntDisAll();                                            /* Disable all interrupts until we are ready to accept them */

    OSInit();                                                   /* Initialize "uC/OS-II, The Real-Time Kernel"              */

     BSP_Init();                                                /* Initialize BSP functions       */
    OSTaskCreateExt(AppTaskStart,                               /* Create the start task                                    */
                    (void *)0,
                    (OS_STK *)&AppTaskStartStk[APP_TASK_START_STK_SIZE - 1],
                    APP_TASK_START_PRIO,
                    APP_TASK_START_PRIO,
                    (OS_STK *)&AppTaskStartStk[0],
                    APP_TASK_START_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if OS_TASK_NAME_SIZE > 13
    OSTaskNameSet(APP_TASK_START_PRIO, "Startup", &err);
#endif

    OSStart();                                                  /* Start multitasking (i.e. give control to uC/OS-II)       */
}

/*
*********************************************************************************************************
*                                          STARTUP TASK
*
* Description : This is an example of a startup task.  As mentioned in the book's text, you MUST
*               initialize the ticker only once multitasking has started.
*
* Arguments   : p_arg   is the argument passed to 'AppStartTask()' by 'OSTaskCreate()'.
*
* Returns     : none
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/

static  void  AppTaskStart (void *p_arg)
{
    CPU_INT08U  i;
    CPU_INT16U  delay;
	CPU_INT08U errstr[6] = "error\0";  // Initialise String
    


    (void)p_arg;

    BSP_Init();                                                 /* Initialize BSP functions                                 */

#if OS_TASK_STAT_EN > 0
    OSStatInit();                                               /* Determine CPU capacity                                   */
#endif
    keyscan_4x4_init();
    
    LED_Off(0);                                                 /* Turn OFF all the LEDs                                    */

		
    MyLEDSem=OSSemCreate(1);                                           /* create a seamaphore  */      
	if(MyLEDSem==(OS_EVENT *) 0) 
	GLCD_WriteString(FT_X*3,2, &errstr[0]);
	                                                                                  
    AppTaskCreate();                                            /* Create application tasks                                 */
    
	while (DEF_TRUE) {                                          /* Task body, always written as an infinite loop.           */
            OSTimeDlyHMSM(0, 0, 30, 0);                         /* Task is not need anymore and could be deleted */
    }
}

/*
*********************************************************************************************************
*                                         LED Flash TASK
*
* Description : This task flashes the LEDs.
*
* Arguments   : p_arg   is the argument passed to 'AppStartUserIF()' by 'OSTaskCreate()'.
*
* Returns     : none
*********************************************************************************************************
*/

static  void  AppTaskLEDFlash (void *p_arg)
{
   CPU_INT08U      err;
 
    (void)p_arg;
   
    while (DEF_TRUE) {
		OSSemPend(MyLEDSem, 0, &err);  // Adding of the Seamaphore
		if(err==OS_NO_ERR)
		{
			LED_On(0);
			OSSemPost(MyLEDSem);
		}
	   
       OSTimeDlyHMSM(0, 0, 0, 300); 
	   	OSSemPend(MyLEDSem, 0, &err);
		if(err==OS_NO_ERR)
		{
			LED_Off(0);
			OSSemPost(MyLEDSem);  //release the Seamaphore
		}  
       
        OSTimeDlyHMSM(0, 0, 0, 300);
	     

    }

}

/*
*********************************************************************************************************
*                                         LED Read TASK
*
* Description : This task reads the status of the LED.
*
* Arguments   : p_arg   is the argument passed to 'AppStartUserIF()' by 'OSTaskCreate()'.
*
* Returns     : none
*********************************************************************************************************
*/

static  void  AppTaskLEDRead (void *p_arg)
{
    CPU_INT08U ledone, ledtwo, err;
    CPU_INT32U eqcnt, uneqcnt;      // Count variables
    CPU_INT08U eqstr[9]   = "Eq 00000\0";  // Initialise String
    CPU_INT08U uneqstr[9] = "Nq 00000\0";  // Initialise String
    
    eqcnt = 0;                      // Initialise count variables
    uneqcnt = 0;                    // Initialise count variables
    GLCD_WriteString(FT_X*3, 4, &eqstr[0]);//DispStr(0, 0, &eqstr[0]);       // Display initialised string
    GLCD_WriteString(FT_X*3, 6, &uneqstr[0]);//DispStr(1, 0, &uneqstr[0]);     // Display initialised string
    
    
    (void)p_arg;
   
     while (DEF_TRUE) {
		 OSSemPend(MyLEDSem, 0,&err);
		 if(err==OS_NO_ERR)
		 {
			 
            ledone = P5 & 0x0F;          // Read LED status
            OSTimeDlyHMSM(0, 0, 0, 100); // Delay for 100ms  
            ledtwo = P5 & 0x0F;          // Read LED status
		    OSSemPost(MyLEDSem);
			}
       if (ledone == ledtwo){       // Are both reads the same?
           eqcnt++;                 // Yes, increment counter
           if (eqcnt > 99999)       // Check for overflow (5 digits)
                eqcnt = 0;
           AppFormatDec(&eqstr[3], eqcnt, 5);   // Convert into string
           GLCD_WriteString(FT_X*3, 4, &eqstr[0]);//DispStr(0, 0, &eqstr[0]);// Display result
       }
       else {
           uneqcnt++;               // Not the same, increment unequal counter
           if (uneqcnt > 99999)     // Check for overflow (5 digits)
                uneqcnt = 0;
           AppFormatDec(&uneqstr[3], uneqcnt, 5);// Convert into string
           GLCD_WriteString(FT_X*3, 6, &uneqstr[0]);//DispStr(1, 0, &uneqstr[0]);// Display result
       }
       
       OSTimeDlyHMSM(0, 0, 1, 0);   // Delay for 1 second

    } // End of While

}


/*
*********************************************************************************************************
*                                      CREATE APPLICATION TASKS
*
* Description:  This function creates the application tasks.
*
* Arguments  :  none
*
* Returns    :  none
*********************************************************************************************************
*/

static  void  AppTaskCreate (void)
{
    CPU_INT08U      err;


    OSTaskCreateExt(AppTaskLEDFlash,
                    (void *)0,
                    (OS_STK *)&AppTaskLEDFlashStk[APP_TASK_LED_FLASH_STK_SIZE - 1],
                    APP_TASK_LED_FLASH_PRIO,
                    APP_TASK_LED_FLASH_PRIO,
                    (OS_STK *)&AppTaskLEDFlashStk[0],
                    APP_TASK_LED_FLASH_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if OS_TASK_NAME_SIZE > 13
    OSTaskNameSet(APP_TASK_LED_FLASH_PRIO, "LED Flash", &err);
#endif

    OSTaskCreateExt(AppTaskLEDRead,
                    (void *)0,
                    (OS_STK *)&AppTaskLEDReadStk[APP_TASK_LED_READ_STK_SIZE - 1],
                    APP_TASK_LED_READ_PRIO,
                    APP_TASK_LED_READ_PRIO,
                    (OS_STK *)&AppTaskLEDReadStk[0],
                    APP_TASK_LED_READ_STK_SIZE,
                    (void *)0,
                    OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR);

#if OS_TASK_NAME_SIZE > 13
    OSTaskNameSet(APP_TASK_LED_READ_PRIO, "LED Read", &err);
#endif

}

/*
*********************************************************************************************************
*                                      FORMAT A DECIMAL VALUE
*
* Description: This function converts a decimal value to ASCII (with leading zeros)
* Arguments  : s       is a pointer to the destination ASCII string
*              value   is the value to convert (assumes an unsigned value)
*              digits  is the desired number of digits
*********************************************************************************************************
*/

static  void  AppFormatDec (CPU_INT08U *s, CPU_INT32U value, CPU_INT08U digits)
{
    CPU_INT08U      i;
    CPU_INT32U      mult;


    mult        = 1;
    for (i = 0; i < (digits - 1); i++) {
        mult   *= 10;
    }
    while (mult > 0) {
        *s++    = value / mult + '0';
        value  %= mult;
        mult   /= 10;
    }
    *s          = 0;
}
ired number of digits
*********************************************************************************